@@ -1,6 +1,6 @@
 library(uGMAR)
 context("functions used in the genetic algorithm")
-# Note that functions creating random parameters are not covered
+# Note that functions creating random parameters are not covered, nor is GAfit
 
 params12 <- c(1.0, 0.9, 0.25, 4.5, 0.7, 3.0, 0.8)
 params22t <- c(1.4, 0.8, 0.05, 0.27, 3.5, 0.9, -0.18, 3.1, 0.7, 203, 3) # StMAR
@@ -29,23 +29,30 @@ params23gsr <- c(1, 2, 3, 0.5, 0.05, 1, 2, 3, 0.4, 0.4, 20, 30) # M1=1, M2=2
 test_that("extractRegime extracts the right regime", {
   expect_equal(extractRegime(2, c(1, 2), params23gsr, model="G-StMAR", restricted=TRUE, regime=1), c(1, 1))
   expect_equal(extractRegime(2, c(1, 2), params23gsr, model="G-StMAR", restricted=TRUE, regime=3), c(3, 3, 30))
+  expect_equal(extractRegime(2, c(1, 2), params23gsr, model="G-StMAR", restricted=TRUE, regime=3, with_dfs=FALSE), c(3, 3))
   expect_equal(extractRegime(2, c(1, 1), params22gs, model="G-StMAR", regime=2), c(2, 0.2, 0.2, 2, 10))
+  expect_equal(extractRegime(2, c(1, 1), params22gs, model="G-StMAR", regime=2, with_dfs=FALSE), c(2, 0.2, 0.2, 2))
   expect_equal(extractRegime(2, c(1, 1), params22gs, model="G-StMAR", regime=1), c(1, 0.1, 0.1, 1))
 
   expect_equal(extractRegime(1, 2, params12, regime=1), c(1.00, 0.90, 0.25))
   expect_equal(extractRegime(2, 2, params22t, model="StMAR", regime=2), c(3.50, 0.90, -0.18, 3.10, 3.00))
+  expect_equal(extractRegime(2, 2, params22t, model="StMAR", regime=2, with_dfs=FALSE), c(3.50, 0.90, -0.18, 3.10))
   expect_equal(extractRegime(2, 3, params23, regime=3), c(7.20, 0.30, -0.01, 0.10))
   expect_equal(extractRegime(1, 2, params12tr, model="StMAR", restricted=TRUE, regime=2), c(0.96, 5.80, 272.00))
+  expect_equal(extractRegime(1, 2, params12tr, model="StMAR", restricted=TRUE, regime=2, with_dfs=FALSE), c(0.96, 5.80))
   expect_equal(extractRegime(2, 3, params23r,  restricted=TRUE, regime=2), c(1.9, 0.7))
   expect_equal(extractRegime(2, 3, params23tr, model="StMAR", restricted=TRUE, regime=3), c(2.1, 3.9, 220.0))
+  expect_equal(extractRegime(2, 3, params23tr, model="StMAR", restricted=TRUE, regime=3, with_dfs=FALSE), c(2.1, 3.9))
 
   expect_equal(extractRegime(2, 1, params21c, model="StMAR", constraints=list(R3), regime=1), c(1.0, 0.9, 1.0, 3.0))
+  expect_equal(extractRegime(2, 1, params21c, model="StMAR", constraints=list(R3), regime=1, with_dfs=FALSE), c(1.0, 0.9, 1.0))
   expect_equal(extractRegime(2, 2, params22c, model="StMAR", constraints=list(R4, R3), regime=2), c(2.0, 0.2, 2.0, 12))
   expect_equal(extractRegime(3, 3, params33c, constraints=list(R2, R2, R1), regime=2), c(2.0, 0.2, 0.2, 0.2, 2.0))
   expect_equal(extractRegime(3, 3, params33c, constraints=list(R2, R2, R1), regime=3), c(3.0, 0.3, -0.3, 3.0))
   expect_equal(extractRegime(2, 1, params21cr, restricted=TRUE, constraints=R3, regime=1), c(1, 1))
   expect_equal(extractRegime(2, 2, params22cr, model="StMAR", restricted=TRUE, constraints=R3, regime=1), c(1, 1, 11))
   expect_equal(extractRegime(2, 2, params22cr, model="StMAR", restricted=TRUE, constraints=R3, regime=2), c(2, 2, 12))
+  expect_equal(extractRegime(2, 2, params22cr, model="StMAR", restricted=TRUE, constraints=R3, regime=2, with_dfs=FALSE), c(2, 2))
   expect_equal(extractRegime(3, 2, params32cr, restricted=TRUE, constraints=R1, regime=1), c(1, 1))
   expect_equal(extractRegime(3, 2, params32cr, restricted=TRUE, constraints=R1, regime=2), c(2, 2))
 })
